/**
 * @file main.c
 * @brief Led verde piscante
 * @author Wu Shin-Ting
 * @date 30/12/2022
 */

//Inclusao dos tipos de dados padronizados pelo ISO C99
#include "stdint.h"

/**
 * @brief computo de numero de iteracoes a partir do multiplo <valor>
 * @param[in] valor multiplo
 * @param[out] i numero de iteracoes
 */
void multiplo_iteracoes (uint32_t valor, uint32_t *i)
{
	*i = valor * 32;
	
	return;
}

/**
 * @brief atraso em multiplo <valor> de 32 interacoes
 * @param[in] valor multiplo de 32
 */
void espera (uint32_t valor)
{
	static uint16_t contador=0;
	static uint32_t contador_offset=5;
	
	uint32_t i;
	

	contador++;
	contador_offset++;
	
	multiplo_iteracoes (valor, &i);
	
	while (i) i--;
}

uint32_t teste;

/**
 * @brief Fluxo de controle principal
 */
int main(void)
{
	//Somente para verificar local de armazenamento das variaveis globais
	teste = 16384;
	
	/**
	 * Configura o modulo SIM: ativacao dos sinais de relogio de PORTB
	 * registrador SIM_SCGC5 mapeado em 0x40048038u (pg. 206)
	 *
	 */
	//(*(unsigned int volatile *) 0x40048038u) =  *(unsigned int *) 0x40048038u | (1<<10);
	//equivalente a:
	*(uint32_t volatile *) 0x40048038u |= (1<<10);

	/**
	 * Configura o modulo PORTB_PCR19 e GPIOB: canal G do led RGB
	 * (pg. 10 em https://www.dca.fee.unicamp.br/cursos/EA871/references/ARM/FRDMKL25Z.pdf)
	 * registrador PORTB_PCR19 mapeado em 0x4004a04c (pg. 179)
	 */
	*(uint32_t volatile *) 0x4004a04cu &= ~0x700; // Zera bits 10, 9 e 8 (MUX) de PTB19
	(*(uint32_t volatile *) 0x4004a04cu) |= 0x00000100; // Seta bit 8 do MUX de PTB19

	/**
	 * Configura o modulo GPIOB: sentido de saida do sinal no pino PTB19
	 * registrador GPIOB_PDDR mapeado em 0x400ff054u (pg. 774)
	 */
	(*(uint32_t volatile *) 0x400ff054u) |= (1<<19);

	/**
	 * Inicializa o estado led G em 1 (ativo baixo)
	 * registrador GPIOB_PCOR mapeado em 0x400ff048u (pg. 776)
	 */
	(*(uint32_t volatile *) 0x400ff048u) = (1<<19);

    /* This for loop should be replaced. By default this loop allows a single stepping. */
    for (;;) {
    	/**
    	 * Reseta o estado led G em 0 (ativo baixo)
    	 * registrador GPIOB_PSOR mapeado em 0x400ff044u (pg. 774)
    	 */
    	(*(uint32_t volatile *) 0x400FF044u) = (1<<19);     	
    	espera (teste);    	
    	/**
    	 * Seta o estado led G em 1 (ativo baixo)
    	 */
		(*(uint32_t volatile *) 0x400FF048u) = (1<<19); 
		espera (teste);
    }

    /* Never leave main */
    return 0;
}

////////////////////////////////////////////////////////////////////////////////
// EOF
////////////////////////////////////////////////////////////////////////////////
